# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/datetime.ipynb (unless otherwise specified).

__all__ = ['datestamp', 'stringToTimestamp']

# Cell
from beartype import beartype

# Cell
from datetime import datetime, timezone, timedelta
@beartype
def datestamp(dt:datetime = datetime.utcnow(), tz=timezone.utc)->int:
  adt:dt = dt.astimezone(tz)
  datestamp = datetime(day=adt.day,month=adt.month,year=adt.year, tzinfo=tz).timestamp()
  return int(datestamp)

# Cell
@beartype
def stringToTimestamp(stringTime:str, formatString:str, timeZone:timezone = timezone.utc)->float:
  return datetime.strptime(stringTime,formatString).replace(tzinfo=timeZone).timestamp()


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/dictUtil.ipynb (unless otherwise specified).

__all__ = ['printDict', 'allKeysInDict', 'filterDt', 'stripDict', 'hashDict', 'saveDictToFile', 'loadDictFromFile',
           'saveStringToFile', 'loadStringFromFile', 'genSchema', 'printYaml', 'loadYaml', 'writeYaml', 'hash_dict',
           'deepUpdate']

# Cell
import yaml
from beartype import beartype

# Cell
def printDict(d: dict, length: int = 10, space=0):
    """
    print dictionary as first length value of values \n
    d: dict: the dictionary to be printed \n
    length: int: if the value is a string, then it will print the first length letter of the value, default = 10 \n
    space: int: the amount of additional space to be added for each nested dictionary, default = 0
    """
    if type(d) != dict:
        print("this is not a dict")
        print(d)
    else:
        for k, v in d.items():
            if type(v) == dict:
                print(f"{' '*space}{k}")
                printDict(v, space=space + 1)
            else:
                print(f"{' '*space}{k} : {v[:length] if type(v)==str else v}")

# Cell
def allKeysInDict(inputDict: dict, keys: list):
    """
    checks whether all the keys given in the list are also keys in the inputDict dictionary \n
    inputDict: dict: the dictionary that will be used to check whether all the keys are in this dictionary \n
    keys: list: the list of keys
    """
    return all(key in inputDict for key in keys)

# Cell
def filterDt(dtDict: dict):
    """
    convert unjsonable datetime object to timestamp in the dictionary, this works for nested dictionary as well \n
    dtDict: dict: the dictionary inputted to convert the datetime object of its value
    """
    from datetime import datetime

    return {
        k: (
            (filterDt(v) if type(v) == dict else v)
            if type(v) != datetime
            else v.timestamp()
        )
        for k, v in dtDict.items()
    }

# Cell
def stripDict(data: dict):
    """
    if the value in the dictionary is a string, it will 'strip' the value to make it more clear \n
    data: dict: the dictionary inputted to be 'stripped'
    """
    return {k: v.strip() if type(v) == str else v for k, v in data.items()}

# Cell
import hashlib, pickle, base64


def hashDict(data: dict, hasher=hashlib.sha1(), encoder=pickle.dumps):
    """
    hashes the dictionary inputted \n
    data: dict: the dictionary inputted to be 'hashed'
    """
    hasher.update(encoder(data))
    rawHash = hasher.digest()
    return base64.b64encode(rawHash).decode()

# Cell
import pickle


def saveDictToFile(data: dict, path: str):
    """
    saves the dictionary to the file directed by the path \n
    data: dict: the dictionary to be saved \n
    path: str: the file path to the file the dictionary is going to be saved
    """
    with open(path, "wb") as f:
        pickle.dump(data, f, protocol=pickle.HIGHEST_PROTOCOL)


def loadDictFromFile(path: str):
    """
    returns the dictionary that is saved in the file \n
    path: str: the path taken to the file of the dictionary
    """
    with open(path, "rb") as f:
        return pickle.load(f)

# Cell
def saveStringToFile(data: str, path: str):
    """
    saves the string to the file directed by the path \n
    data: str: the string to be saved \n
    path: str: the file path to the file the string is going to be saved
    """
    with open(path, "w") as f:
        f.write(data)


def loadStringFromFile(path: str):
    """
    returns the string that is saved in the file \n
    path: str: the path taken to the file of the string
    """
    with open(path, "r") as f:
        return f.read()

# Cell
def genSchema(inputDict: dict, format_="yaml") -> (dict, str):
    """
    generate a json schema from dict \n
    format: return schema in json or yaml, 'both' can be inputted to return a tuple of (json, yaml), output = dict or str, default = 'yaml' \n
    inputDict: dict: the dict inputted to be used to generate the schema
    """
    from genson import SchemaBuilder
    import yaml

    builder = SchemaBuilder()
    builder.add_object(inputDict)
    schema = builder.to_schema()
    if format_ == "yaml":
        return yaml.dump(schema)
    elif format_ == "json":
        return schema
    elif format_ == "both":
        return schema, yaml.dump(schema)
    else:
        return schema, yaml.dump(schema)

# Cell
def printYaml(input_: dict, returnYaml=False) -> (str, None):
    yamlStr: str = yaml.dump(input_)
    print(yamlStr)
    if returnYaml:
        return yamlStr

# Cell
@beartype
def loadYaml(path: str, loader=yaml.FullLoader) -> dict:
    """load yaml string
    input:
      path:str: path of the data to load
    return:
      diet of the yaml

    """
    with open(path) as f:
        r = yaml.load(f.read(), Loader=loader)
    return r

# Cell
@beartype
def writeYaml(path: str, data: dict):
    with open(path, "w") as f:
        f.write(yaml.dump(data))

# Cell
import functools
import json
import ujson
import pickle


def hash_dict(func):
    """Transform mutable dictionnary
    Into immutable
    Useful to be compatible with cache
    """

    class HDict(dict):
        def __hash__(self):
            return hash(ujson.dumps(self, sort_keys=True))

    #           return hash(tuple(frozenset(sorted(self.items()))))
    #           print(self)
    #           return hash(hashDict(self))
    #             return hash(frozenset(self.items()))

    @functools.wraps(func)
    def wrapped(*args, **kwargs):
        args = tuple([HDict(arg) if isinstance(arg, dict) else arg for arg in args])
        kwargs = {k: HDict(v) if isinstance(v, dict) else v for k, v in kwargs.items()}
        return func(*args, **kwargs)

    return wrapped

# Cell
import collections


def deepUpdate(d, u):
    for k, v in u.items():
        if isinstance(d, collections.Mapping):
            if isinstance(v, collections.Mapping):
                r = deepUpdate(d.get(k, {}), v)
                d[k] = r
            else:
                d[k] = u[k]
        else:
            d = {k: u[k]}
    return d


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/shortenLInk.ipynb (unless otherwise specified).

__all__ = ['shorten', 'url']

# Cell
from requests import post
url = 'https://zrm4csjv5a.execute-api.ap-southeast-1.amazonaws.com/Prod/shorten'
def shorten(longLink,url=url):
  '''
  shortens the link inputted in the form of tenxor.sh
  '''
  r = post(url ,json = {'link':longLink})
  return r.json()['shortLink']


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/compress.ipynb (unless otherwise specified).

__all__ = ['compressDict', 'decompressDict', 'compressString', 'decompressString']

# Cell
import gzip, zlib, base64, pickle
from typing import Callable, Optional, List, Dict
from beartype import beartype

# Cell
@beartype
def compressDict(data:(List,Dict),encoder:Callable= pickle.dumps,
                 compressor:Callable=zlib.compress,
                 stringifier:Callable = base64.b64encode)->bytes:
  compressedData =  compressor(encoder(data))
  return stringifier(compressedData)

# Cell
@beartype
def decompressDict( data:(bytes,str),
                   decoder = pickle.loads,
                   decompressor:Callable= zlib.decompress,
                   destringifier:Callable = base64.b64decode)->(List,Dict):
  compressedData = destringifier(data)
  return decoder(decompressor(compressedData))

# Cell
@beartype
def compressString(data:str ,
                 compressor:Callable=zlib.compress,
                 stringifier:Callable = base64.b64encode)->str:
  compressedData =  compressor(data.encode())
  return stringifier(compressedData).decode()

# Cell
@beartype
def decompressString(data:(bytes,str),
                   decompressor:Callable= zlib.decompress,
                   destringifier:Callable = base64.b64decode)->str:
  compressedData = destringifier(data)
  return decompressor(compressedData).decode()


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/wrappers.ipynb (unless otherwise specified).

__all__ = ['add_method', 'add_static_method', 'add_class_method']

# Cell
from functools import wraps # This convenience func preserves name and docstring


# Cell
def add_method(cls):
  '''
  a decorator that allows you to add methods to a class
  '''
  def decorator(func):
      @wraps(func)
      def wrapper(self, *args, **kwargs):
          return func(self, *args, **kwargs)
      setattr(cls, func.__name__, wrapper)
      # Note we are not binding func, but wrapper which accepts self but does exactly the same as func
      return func # returning func means func can still be used normally
  return decorator


# Cell
def add_static_method(cls):
  '''
  a decorator that allows you to add static methods to a class
  '''
  def decorator(func):
    @wraps(func)
    def wrapper( *args, **kwargs):
      return func(*args, **kwargs)
    setattr(cls, func.__name__, wrapper)
  return decorator

# Cell
def add_class_method(cls):
  '''
  a decorator that allows you to add class method to a class
  '''
  def decorator(func):
      @wraps(func)
      @classmethod
      def wrapper(cls, *args, **kwargs):
          return func( cls, *args, **kwargs)
      setattr(cls, func.__name__, wrapper)
      # Note we are not binding func, but wrapper which accepts self but does exactly the same as func
      return func # returning func means func can still be used normally
  return decorator


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/github.ipynb (unless otherwise specified).

__all__ = ['githubGet', 'githubGetYaml', 'githubImportModule']

# Cell
import requests
from requests import get, Response
from beartype import beartype
from typing import Any, Optional

# Cell
@beartype
def githubGet(url:str, token:Optional[str])->Response:
  '''
    get request to github using token
    parameters:
      url:str: github raw file url
      token:str: your personal access token
    response:
      response: requests.Response: response object
  '''
  r = requests.get(url, headers = {"Authorization": f"token {token}"})
  return r

# Cell
@beartype
def githubGetYaml(url:str, token:Optional[str])->dict:
  '''
    get yaml dictionary from github using token
    parameters:
      url:str: github raw file url
      token:str: your personal access token
    response:
      response: dict: yaml dictionary
  '''
  import yaml
  r = githubGet(url, token)
  return yaml.load(r.text,Loader=yaml.FullLoader)


# Cell
@beartype
def githubImportModule(url:str, token:Optional[str], path='/tmp/inputfile', sourceName='schema')->Any:
  import imp
  r = requests.get(url, headers={'Authorization': 'token '+token})
  with open(path, 'w') as f:
    f.write(r.text)
  return imp.load_source(sourceName, path)


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/distance.ipynb (unless otherwise specified).

__all__ = ['getDistanceOsrm', 'getDistBing']

# Cell
from beartype import beartype
from typing import List, Dict, Tuple
import pandas as pd
import yaml, requests

# Cell
@beartype
def getDistanceOsrm(lat1:float, lon1:float, lat2:float, lon2:float,
                    osrmEndpoint = 'http://router.project-osrm.org')->float:
  '''
    get driving distance between 2 points using osrm url
  '''
  url = f"{osrmEndpoint}/route/v1/bike/{lon1},{lat1};{lon2},{lat2}?overview=false"
  r = requests.get(url)
  try:
    distance = r.json()['routes'][0]['distance']
    return float(distance)
  except KeyError as e:
    raise KeyError(f"error getting distance got {r.json()} from urll \n{e}")


# Cell
@beartype
def getDistBing(origin:Tuple[float,float]
                ,destinations:List[Tuple[float,float]],
                 bingApiKey:str = '')->pd.DataFrame:
  '''
    accept a list of tuple origins and destinations, return a dataframe
    input:
      origin :: Tuple[float,float] : (lat,long) of the origin
      destinations:: List[Tuple[float,float]]: [(lat,long),(lat,long)] of the destinations
      bingApiKey:str:: apikey from bing map
    response:
      pd.DataFrame with [dist, dur, lat, long] as indexes, all in float,
          all distances in m and time in s
  '''

  parameters = {
    'origins' : f'{origin[0]},{origin[1]}',
    'destinations' : ';'.join(f'{d[0]},{d[1]}'for d in destinations),
    'travelMode' : 'driving',
    'key': bingApiKey
  }
  url = 'https://dev.virtualearth.net/REST/v1/Routes/DistanceMatrix'
  r =  requests.get(url,params=parameters).json()
  results:pd.DataFrame = pd.DataFrame(r['resourceSets'][0]['resources'][0]['results'])
  destinationsDf:pd.DataFrame = pd.DataFrame(destinations)
  destinationsDf.columns = ['lat', 'long']
  results = pd.concat([results,destinationsDf],axis=1).reindex(
    ['travelDistance','travelDuration', 'lat','long'], axis = 1)
  results.columns  = ['dist', 'dur', 'lat', 'long']
  return results



# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/colab.ipynb (unless otherwise specified).

__all__ = ['saveAwsPw', 'loadAwsPw', 'setUpAws', 'setUpAws2', 'shieldStrings', 'autoSetupAws']

# Cell
from .encrypt import encryptPasswordAes, decryptPasswordAes
from beartype import beartype
from getpass import getpass
from functools import partial
from typing import Tuple, Optional
from getpass import getpass
from pathlib import Path
import subprocess, pickle, os

# Cell
@beartype
def saveAwsPw(awskey:bytes, awssecret:bytes,
              path:str = '/content/drive/MyDrive/.awskey', password:bytes = b''):
  '''
    save your aws password in your
  '''

  # encrypt with aes
  encrypt = partial(encryptPasswordAes, password)
  cryptkey, keynonce = encrypt(awskey)
  cryptsecret, secretnonce = encrypt(awssecret)

  # save file as pickle
  print(f'saving the AES encrypted credentials to {path}')
  with open (path ,'wb') as f:
    pickle.dump((cryptkey, cryptsecret, keynonce, secretnonce),f)


# Cell
@beartype
def loadAwsPw(path = '/content/drive/MyDrive/.awskey',
              password:Optional[bytes] = b'' )->Optional[Tuple[bytes,bytes]]:
  '''
    load key and secret from path
  '''
  print( f'loading password from {path}')

  with open (path, 'rb') as f:
    cryptkey, cryptsecret, keynonce, secretnonce = pickle.load(f)
  decryptedKey = decryptPasswordAes(cryptkey,password,keynonce)
  decryptedSecret = decryptPasswordAes(cryptsecret, password, secretnonce)
  return decryptedKey, decryptedSecret


# Cell
@beartype
def setUpAws(awsKey:str, awsSecret:str,
             profile:Optional[str] = None, region:str = 'ap-southeast-1')->Tuple[str,str,str]:
  '''
    this code generate a string to be executed to the shell
    you can do this with, however, its not a bad idea to check before executing

    (exec(c) for c in setUpAws(...))

    response:
      (setupKey, setupSecret, setupRegion)

  '''
  if profile:  profileParameter = f'--profile {profile} '
  else: profileParameter = ''

  setupPrefix = f'aws configure {profileParameter}'
  setupKey = f'{setupPrefix}set aws_access_key_id {awsKey}'
  setupSecret = f'{setupPrefix}set aws_secret_access_key {awsSecret}'
  setupRegion = f'{setupPrefix}set default.region {region}'
  return (setupKey, setupSecret, setupRegion)



# Cell
@beartype
def setUpAws2(awsKey:str, awsSecret:str,
             profile:Optional[str] = None, region:str = 'ap-southeast-1')->Tuple[str,str]:

  configString = f'''[{profile or 'default'}]\nregion = {region}\noutput = json'''

  credentialString =  f'''[{profile or 'default'}]\naws_access_key_id = {awsKey}\naws_secret_access_key = {awsSecret}'''
  return configString, credentialString

# Cell
def shieldStrings(strings):
  r = ''
  for line in strings.split('\n'):
    words = line.split(' ')
    r += ' '.join(words[:2] + [f'{"*" * len(words[2])}' if len(line.split(' ')) == 3 else '']) + '\n'
  return r

# Cell
@beartype
def autoSetupAws(path:str, profile:Optional[str]=None, region:str='ap-southeast-1', mockup:bool = False, password:Optional[bytes] = None):
  key, secret = loadAwsPw(path, password=password) # key and secret in bytes
  try:
    key, secret = key.decode(), secret.decode()
  except:
    print('cant decode key and secret, maybe the password is wrong')
    return

  config, creds = setUpAws2(awsKey=key,
                          awsSecret=secret,
                          profile=profile,
                          region=region)
  print('config is :', shieldStrings(config), shieldStrings(creds), sep='\n')


  # check if reunning in colab
  isColab =  'google.colab' in str(get_ipython())
  if isColab:
    print('Running on CoLab')
  else:
    print('Not running on colab')

  awsPath = f'{Path.home()}/.aws'
  print(f'awspath is {awsPath}')

  if isColab:
    print('creating a folder')
    try:
      os.mkdir(awsPath)
    except:
      print('path exist')
    awsPath = f'{Path.home()}/.aws'
    print('writing creds files')
    with open (f'{awsPath}/credentials', 'w') as f:
      f.write(creds)
    with open (f'{awsPath}/config', 'w') as f:
      f.write(config)
  else:
    print('not colab')


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/awsUtil.ipynb (unless otherwise specified).

__all__ = ['invokeLambdaWithApigatewayProxy']

# Cell
import json
import boto3
from pprint import pprint


def invokeLambdaWithApigatewayProxy(
    payload, function_name, region_name="ap-southeast-1", debug=False, queryparams={}
):
    """
    Invokes an AWS Lambda function through the API Gateway proxy integration, with support for query parameters.

    Args:
        payload (dict): The input data to send to the Lambda function. This will be
            serialized to JSON and included in the 'body' of the request.
        function_name (str): The name of the Lambda function to invoke.
        region_name (str, optional): The AWS region where the Lambda function is deployed.
            Defaults to 'ap-southeast-1'.
        debug (bool, optional): If True, prints the raw response payload for debugging purposes.
            Defaults to False.
        query_params (dict, optional): A dictionary of query parameters to include in the request,
            as if making a GET request. These parameters are serialized and included in the request.

    Returns:
        dict: The parsed response from the Lambda function, including the 'body' key
            containing the JSON-decoded result.

    Raises:
        KeyError: If the Lambda function's response does not include a 'body' key.
        Exception: If there is an error decoding the Lambda response payload.
    """
    lambda_client = boto3.client("lambda", region_name=region_name)
    payload_json = json.dumps(
        {"body": json.dumps(payload), "queryStringParameters": queryparams}
    )
    response = lambda_client.invoke(
        FunctionName=function_name,
        InvocationType="RequestResponse",  # RequestResponse waits for the response, Event is async
        Payload=payload_json.encode("utf-8"),
    )
    responsePayload = response["Payload"].read().decode("utf-8")
    try:
        result = json.loads(responsePayload)
        body = json.loads(result["body"])
        result["body"] = body
        if debug:
            pprint(responsePayload)
        return result
    except KeyError as e:
        print("there is a keyerror perhaps the function failed", e)
        print("result is")
        pprint(responsePayload)
        raise (e)
    except Exception as e:
        print("decoding failed", e)
        print(result)
        pprint(responsePayload)
        raise (e)


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/jupyter.ipynb (unless otherwise specified).

__all__ = ['displayHtml']

# Cell
def displayHtml(html):
    from IPython.display import display, HTML
    display(HTML(html))


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/yamlUtil.ipynb (unless otherwise specified).

__all__ = ['loadYamlUrl', 'loadYaml', 'saveYaml']

# Cell
from beartype import beartype
import yaml, requests

# Cell
def loadYamlUrl(url:str, headers:dict = {}, loader=yaml.FullLoader)->dict:
  '''
    load yaml from url as a dictionary for example:
      testUrl = 'https://raw.githubusercontent.com/thanakijwanavit/villaMasterSchema/dev/coupon/testData/buyXGetY.yaml'
      loadYamlUrl(testUrl)
    input:
      url:str: url of the file
      headers:Optional[dict]:headers
      loader:yaml.Loader: optional yaml loader default is yaml.FullLoader
    response:
      dict: yaml load of the file
  '''
  r = requests.get(url, headers=headers).content
  return yaml.load(r, Loader = loader)


# Cell
@beartype
def loadYaml(path:str)->dict:
  return yaml.load(open(path).read(), Loader=yaml.FullLoader)

# Cell
@beartype
def saveYaml(item:dict, path:str)->bool:
  with open(path,'w') as f:
    f.write(yaml.dump(item))
  return True


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/exception.ipynb (unless otherwise specified).

__all__ = ['errorString']

# Cell
import traceback

# Cell
def errorString():
  '''
  returns a string of the exception information
  '''
  return traceback.format_exc()


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/cloudfront.ipynb (unless otherwise specified).

__all__ = ['invalidate']

# Cell
import boto3, time
from beartype import beartype

# Cell
@beartype
def invalidate(distributionId:str, path:str='/*')->str:
  '''
    create a cloudfront invalidation
    parameters:
      distributionId:str: distribution id of the cf distribution
      path:str: path to invalidate, can use wildcard eg. "/*" means all path
    response:
      invalidationId:
  '''
  cf = boto3.client('cloudfront')
  # Create CloudFront invalidation
  res = cf.create_invalidation(
      DistributionId=distributionId,
      InvalidationBatch={
          'Paths': {
              'Quantity': 1,
              'Items': [
                  path
              ]
          },
          'CallerReference': str(time.time()).replace(".", "")
      }
  )
  invalidation_id = res['Invalidation']['Id']
  return invalidation_id


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/jsonUtil.ipynb (unless otherwise specified).

__all__ = ['loadJson', 'saveJson']

# Cell
import json


def loadJson(path: str) -> dict:
    with open(path, "r") as f:
        return json.load(f)

# Cell
def saveJson(path: str, data: dict):
    with open(path, "w") as f:
        json.dump(data, f)
    return True


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/timer.ipynb (unless otherwise specified).

__all__ = ['logger', 'Timer', 'start_timer', 'reset_timer', 'print_time', 'print_reset', 'log_time', 'log_reset']

# Cell
from .wrappers import add_class_method,add_method,add_static_method
from datetime import datetime, timedelta
import logging
logger = logging.getLogger(name='timer')

# Cell
class Timer:
  '''
  This is the class that will be used for the timer
  '''
  def __init__(self):
    self.start_timer()
  pass


# Cell
@add_method(Timer)
def start_timer(self):
  '''
  this method sets the starting time t0 to the current time
  '''
  self.t0 = datetime.now()
@add_method(Timer)
def reset_timer(self):
  '''
  this method resets t0 to the current time
  '''
  self.t0 = datetime.now()

# Cell
@add_method(Timer)
def print_time(self, description = 'fuction took'):
  '''
  this method subtracts the current time by t0 and prints the value in seconds to find out time between start timer and this method \n
  description: str: this is the string to be added before the value of time taken, default = 'function took'
  '''
  t1:timedelta = datetime.now() - self.t0
  print(f'{description} :{t1.total_seconds()} s')
  return t1.total_seconds()

# Cell
@add_method(Timer)
def print_reset(self, description = 'function took'):
  '''
  print and reset timer \n
  description: str: description of the item to print
  '''

  r = self.print_time(description = description)
  self.reset_timer()
  return r

# Cell
@add_method(Timer)
def log_time(self, description = 'fuction took', logger = logger.debug):
  '''
  this method subtracts the current time by t0 and prints the value in seconds to find out time between start timer and this method \n
  parameters:
    description: str: this is the string to be added before the value of time taken, default = 'function took'
    logger: callable: a callable for logging, default: logging.debug
  response:
    time:float:: time in second
  '''
  t1:timedelta = datetime.now() - self.t0
  logger(f'{description} :{t1.total_seconds()} s')
  return t1.total_seconds()

# Cell
@add_method(Timer)
def log_reset(self, description = 'function took', logger = logger.debug):
  '''
  print and reset timer \n
  parameters:
    description:  str: description of the item to print
    logger: callable: a callable for logging, default: logging.debug
  response:
    time:float:: time in second

  '''

  r = self.log_time(description = description,logger=logger)
  self.reset_timer()
  return r